@@ -9,17 +9,15 @@ def _remove_date(brief:str) -> str:
     return re.sub(r',? (:?january|february|march|april|may|june|july|august|september|october|november|december) [0-9]{1,2}.*$', '', brief, flags = re.IGNORECASE)
 
 MANUAL_OVERRIDE = [
+    (r'^$', []),
     ("1. Brief, BRIEF OF AMICI CURIAE NATIONAL ASSOCIATION OF CRIMINAL DEFENSE LAWYERS AND FAMILIES AGAINST MANDATORY MINIMUMS FOUNDATION AND ASSOCIATION OF FEDERAL DEFENDERS IN SUPPORT OF PETITIONERTHE NATIONAL ASSOCIATION OF CRIMINAL DEFENSE LAWYERS and FAMILIES AGAINST MANDATORY MINIMUMS FOUNDATION and THE ASSOCIATION OF FEDERAL DEFENDERS", ['NATIONAL ASSOCIATION OF CRIMINAL DEFENSE LAWYERS', 'FAMILIES AGAINST MANDATORY MINIMUMS FOUNDATION', 'ASSOCIATION OF FEDERAL DEFENDERS'])
 ]
 
 def amici(brief:str) -> list:
     for member, result in MANUAL_OVERRIDE:
-        if member in brief:
+        if re.match(member, brief):
             return result
-    else:
-        return list(_amici(brief))
 
-def _amici(brief:str) -> iter:
     _amicus_regex = re.compile(r'(?:amicus brief|amici brief|amici curiae|amicus curiae|motion for leave to file and brief)(?: of)?', flags = re.IGNORECASE)
     amici_section = _remove_date(brief)
     amici_section = re.sub(r'[0-9]+\. +Brief,', '', amici_section, flags = re.IGNORECASE)
@@ -33,19 +31,15 @@ def _amici(brief:str) -> iter:
 
     amici_section = re.sub(r' in support of .*', '', amici_section, flags = re.IGNORECASE)
 
-#   if re.search(r' inc[^a-z]', amici_section, flags = re.IGNORECASE):
-#       buffer = 5
-#   elif ',' in amici_section and ' and ' in amici_section.lower():
-#       buffer = 0
-#   else:
-#       buffer = 0
-    buffer = 0
+    # Inc.The
+    amici_section = re.sub(r'([^ ])The ', r'\1, The ', amici_section)
 
+    onlycomma = r'(?:,| and the| and other) '
     l = amici_section.lower()
     if l.count(';') > 0:
         _regex = r'; '
     elif l.count(',') > 3 or ', and' in l or l.count(',') > l.count('and') or l.count(',') == l.count(', inc'):
-        _regex = r'(?:,| and the| and other) '
+        _regex = onlycomma 
     else:
         _regex = r'(?:,| and) '
     amicus_separator = re.compile(_regex, flags = re.IGNORECASE)
@@ -63,35 +57,45 @@ def _amici(brief:str) -> iter:
             return r
 
     # Clean twice
-    results = map(clean, map(clean, _amicus(unidecode(amici_section), amicus_separator, buffer, 0)))
-    slider = window(chain(['  '], results, ['  ']), n = 3)
+    results = map(clean, map(clean, _amicus(unidecode(amici_section), amicus_separator, 0)))
+    slider = window(chain([''], results, ['']), n = 3)
+    out = []
     for previous_result, current_result, next_result in slider:
         if re.match(r'^(|as|amic(i|us) curiae)$', current_result, flags = re.IGNORECASE):
             pass
         elif re.match(r'^[^a-z]{0,2}(inc|jr)[^a-z]{0,2}', next_result, flags = re.IGNORECASE):
-            yield current_result + ', ' + next_result
+            out.append(current_result + ', ' + next_result)
             next(slider)
         elif re.match(r'^et al\.?,?$', next_result, flags = re.IGNORECASE):
-            yield current_result + ', ' + next_result
+            out.append(current_result + ', ' + next_result)
             next(slider)
         else:
-            yield current_result
+            out.append(current_result)
 
-def _amicus(brief:str, amicus_separator, buffer:int, start:int) -> iter:
+    if len(out) >= 3 and _regex == onlycomma:
+        out = out[:-1] + re.split(r' and ', out[-1], flags = re.IGNORECASE)
+
+    def finalize(result):
+        result = re.sub(r'^ ?(amici|of )', '', result, flags = re.IGNORECASE)
+        result = re.sub(r' (on behalf of).*$', '', result, flags = re.IGNORECASE)
+        result = re.sub(r',$', '', result)
+        return result
+    return list(map(finalize, out))
+
+def _amicus(brief:str, amicus_separator, start:int) -> iter:
     match = re.search(amicus_separator, brief[start:])
-    buffered_start = max(0, start - buffer)
     if match != None:
-        buffered_end = start + match.start() + buffer
-        result = brief[buffered_start:buffered_end]
+        end = start + match.start()
+        result = brief[start:end]
 
         if result.count(' ') == 0:
-            if ' and' == brief[buffered_end+1:buffered_end + 5]:
+            if ' and' == brief[end+1:end + 5]:
                 # Probably the beginning of an amicus like
                 # "Discrimination and National Security Initiative"
-                nextmatch = re.search(_amicus_separator, brief[start + match.end()])
+                nextmatch = re.search(amicus_separator, brief[start + match.end()])
                 if nextmatch != None:
                     # Replace the result.
-                    result = brief[buffered_start:start + nextmatch.end() + buffer]
+                    result = brief[start:start + nextmatch.end()]
 
         if re.search(r'[a-z]{4}\.', result):
             match = re.match(r'^(.*[a-z]{4})\.(.*)$', result)
@@ -101,16 +105,16 @@ def _amicus(brief:str, amicus_separator, buffer:int, start:int) -> iter:
             pass
         else:
             yield result
-        child = _amicus(brief, amicus_separator, buffer, start + match.end())
+        child = _amicus(brief, amicus_separator, start + match.end())
         if child != None:
             yield from child
     else:
-        yield brief[buffered_start:]
+        yield brief[start:]
 
-def brief_number(brief:str) -> int:
-    return int(re.sub('[^0-9].+$', '', brief))
+def brief_number(brief:str) -> str:
+    return re.sub('[^0-9].+$', '', brief)
 
-def posture(brief:str) -> int:
+def posture(brief:str) -> str:
     codes = [
         (0, ('Neither party',)), # maybe change it to just "Neither"?
         (1, ('Petitioner', 'Appellant', 'Reversal')),
@@ -126,6 +130,6 @@ def posture(brief:str) -> int:
                 code_phrase_observations[code] += 1
 
     if len(code_phrase_observations.keys()) == 1:
-        return list(code_phrase_observations.keys())[0]
+        return str(list(code_phrase_observations.keys())[0])
     else:
         return ''
